aboutsummaryrefslogtreecommitdiff
path: root/applets/busybox.c
diff options
context:
space:
mode:
Diffstat (limited to 'applets/busybox.c')
-rw-r--r--applets/busybox.c129
1 files changed, 44 insertions, 85 deletions
diff --git a/applets/busybox.c b/applets/busybox.c
index ee74b4c18..420c9c0e5 100644
--- a/applets/busybox.c
+++ b/applets/busybox.c
@@ -9,7 +9,6 @@
#include <locale.h>
#endif
-int been_there_done_that = 0; /* Also used in applets.c */
const char *bb_applet_name;
#ifdef CONFIG_FEATURE_INSTALLER
@@ -32,17 +31,6 @@ static const char* const install_dir[] = {
/* abstract link() */
typedef int (*__link_f)(const char *, const char *);
-/*
- * Where in the filesystem is this busybox?
- * [return]
- * malloc'd string w/ full pathname of busybox's location
- * NULL on failure
- */
-static inline char *busybox_fullpath(void)
-{
- return xreadlink("/proc/self/exe");
-}
-
/* create (sym)links for each applet */
static void install_links(const char *busybox, int use_symbolic_links)
{
@@ -72,41 +60,27 @@ int main(int argc, char **argv)
{
const char *s;
- bb_applet_name = argv[0];
-
- if (bb_applet_name[0] == '-')
- bb_applet_name++;
+ bb_applet_name=argv[0];
+ if (*bb_applet_name == '-') bb_applet_name++;
+ for (s = bb_applet_name; *s ;)
+ if (*(s++) == '/') bb_applet_name = s;
- for (s = bb_applet_name; *s != '\0';) {
- if (*s++ == '/')
- bb_applet_name = s;
- }
-
-#ifdef CONFIG_LOCALE_SUPPORT
-#ifdef CONFIG_INIT
- if(getpid()!=1) /* Do not set locale for `init' */
-#endif
- {
+ /* Set locale for everybody except `init' */
+ if(ENABLE_LOCALE_SUPPORT && (!ENABLE_INIT || getpid()==1))
setlocale(LC_ALL, "");
- }
-#endif
run_applet_by_name(bb_applet_name, argc, argv);
bb_error_msg_and_die("applet not found");
}
-
int busybox_main(int argc, char **argv)
{
- int col = 0, len, i;
-
-#ifdef CONFIG_FEATURE_INSTALLER
/*
* This style of argument parsing doesn't scale well
* in the event that busybox starts wanting more --options.
* If someone has a cleaner approach, by all means implement it.
*/
- if (argc > 1 && (strcmp(argv[1], "--install") == 0)) {
+ if (ENABLE_FEATURE_INSTALLER && argc > 1 && !strcmp(argv[1], "--install")) {
int use_symbolic_links = 0;
int rc = 0;
char *busybox;
@@ -119,7 +93,7 @@ int busybox_main(int argc, char **argv)
}
/* link */
- busybox = busybox_fullpath();
+ busybox = xreadlink("/proc/self/exe");
if (busybox) {
install_links(busybox, use_symbolic_links);
free(busybox);
@@ -128,60 +102,45 @@ int busybox_main(int argc, char **argv)
}
return rc;
}
-#endif /* CONFIG_FEATURE_INSTALLER */
-
- argc--;
-
- /* If we've already been here once, exit now */
- if (been_there_done_that == 1 || argc < 1) {
- const struct BB_applet *a = applets;
- int output_width = 60;
-
-#ifdef CONFIG_FEATURE_AUTOWIDTH
- /* Obtain the terminal width. */
- get_terminal_width_height(0, &output_width, NULL);
- /* leading tab and room to wrap */
- output_width -= 20;
-#endif
- printf("%s\n\n"
- "Usage: busybox [function] [arguments]...\n"
- " or: [function] [arguments]...\n\n"
- "\tBusyBox is a multi-call binary that combines many common Unix\n"
- "\tutilities into a single executable. Most people will create a\n"
- "\tlink to busybox for each function they wish to use and BusyBox\n"
- "\twill act like whatever it was invoked as!\n"
- "\nCurrently defined functions:\n", bb_msg_full_version);
-
- while (a->name != 0) {
- col +=
- printf("%s%s", ((col == 0) ? "\t" : ", "),
- (a++)->name);
- if (col > output_width && a->name != 0) {
- printf(",\n");
- col = 0;
+ /* Deal with --help. (Also print help when called with no arguments) */
+
+ if (argc==1 || !strcmp(argv[1],"--help") ) {
+ if (argc>2) run_applet_by_name(bb_applet_name=argv[2], argc, argv);
+ else {
+ const struct BB_applet *a;
+ int col, output_width;
+
+ if (ENABLE_FEATURE_AUTOWIDTH) {
+ /* Obtain the terminal width. */
+ get_terminal_width_height(0, &output_width, NULL);
+ /* leading tab and room to wrap */
+ output_width -= 20;
+ } else output_width = 60;
+
+ printf("%s\n\n"
+ "Usage: busybox [function] [arguments]...\n"
+ " or: [function] [arguments]...\n\n"
+ "\tBusyBox is a multi-call binary that combines many common Unix\n"
+ "\tutilities into a single executable. Most people will create a\n"
+ "\tlink to busybox for each function they wish to use and BusyBox\n"
+ "\twill act like whatever it was invoked as!\n"
+ "\nCurrently defined functions:\n", bb_msg_full_version);
+
+ col=0;
+ for(a = applets; a->name;) {
+ col += printf("%s%s", (col ? ", " : "\t"), (a++)->name);
+ if (col > output_width && a->name) {
+ printf(",\n");
+ col = 0;
+ }
}
+ printf("\n\n");
+ exit(0);
}
- printf("\n\n");
- exit(0);
- }
-
- /* Flag that we've been here already */
- been_there_done_that = 1;
-
- /* Move the command line down a notch */
- /* Preserve pointers so setproctitle() works consistently */
- len = argv[argc] + strlen(argv[argc]) - argv[1];
- memmove(argv[0], argv[1], len);
- memset(argv[0] + len, 0, argv[1] - argv[0]);
-
- /* Fix up the argv pointers */
- len = argv[1] - argv[0];
- memmove(argv, argv + 1, sizeof(char *) * (argc + 1));
- for (i = 0; i < argc; i++)
- argv[i] -= len;
-
- return (main(argc, argv));
+ } else run_applet_by_name(bb_applet_name=argv[1], argc-1, argv+1);
+
+ bb_error_msg_and_die("applet not found");
}
/*